import dom from './dom';
import SerializedHighlight from './SerializedHighlight';
export const FOCUS_CSS = 'focus';
export default class Highlight {
    constructor(arg, arg2, content) {
        this._elements = [];
        if (arg instanceof Range) {
            // TODO - something more random here
            this._id = (new Date()).getTime().toString();
            this._range = arg;
            this._content = arg2;
        }
        else if (content) {
            this._id = arg;
            this._range = arg2;
            this._content = content;
        }
        else {
            throw new Error('invalid constructor arguments');
        }
    }
    get id() {
        return this._id;
    }
    get content() {
        return this._content;
    }
    get range() {
        return this._range;
    }
    set elements(elements) {
        if (this.elements.length > 0) {
            throw new Error(`Hightlight elements aren't reloadable`);
        }
        this._elements = elements;
    }
    get elements() {
        return this._elements;
    }
    isAttached() {
        // TODO - check and see if these are in the dom
        return this.elements.length > 0;
    }
    scrollTo(handler) {
        if (!this.isAttached()) {
            return this;
        }
        else if (handler) {
            handler(this.elements);
        }
        else {
            this.elements[0].scrollIntoView();
        }
        return this;
    }
    focus() {
        this.elements.forEach((el) => el.classList.add(FOCUS_CSS));
        return this;
    }
    intersects(range) {
        if (!range) {
            return false;
        }
        return this.range.compareBoundaryPoints(Range.START_TO_END, range) !== -1
            && this.range.compareBoundaryPoints(Range.END_TO_START, range) !== 1;
    }
    serialize(referenceElement) {
        referenceElement = referenceElement || dom(this.range.commonAncestorContainer).closest('[id]');
        return new SerializedHighlight(Object.assign({ content: this.content, id: this.id }, SerializedHighlight.defaultSerializer(this.range, referenceElement)));
    }
}
//# sourceMappingURL=Highlight.js.map