import Highlight, { FOCUS_CSS } from './Highlight';
import injectHighlightWrappers from './injectHighlightWrappers';
import { rangeContentsString } from './rangeContents';
import removeHighlightWrappers from './removeHighlightWrappers';
import { getRange, snapSelection } from './selection';
import SerializedHighlight from './SerializedHighlight';
export default class Highlighter {
    constructor(container, options = {}) {
        this.highlights = {};
        this.onMouseup = () => {
            const selection = this.document.getSelection();
            if (!selection) {
                return;
            }
            if (selection.isCollapsed) {
                this.onClick(selection);
            }
            else {
                this.onSelect(selection);
            }
        };
        this.container = container;
        this.options = options;
        this.container.addEventListener('mouseup', this.onMouseup);
    }
    unmount() {
        this.container.removeEventListener('mouseup', this.onMouseup);
    }
    erase(highlight) {
        removeHighlightWrappers(highlight);
        delete this.highlights[highlight.id];
    }
    highlight(highlight) {
        if (highlight instanceof SerializedHighlight && highlight.isLoadable(this)) {
            return this.highlight(highlight.load(this));
        }
        else if (highlight instanceof Highlight) {
            this.highlights[highlight.id] = highlight;
            injectHighlightWrappers(highlight, this.options);
        }
    }
    getHighlight(id) {
        return this.highlights[id];
    }
    getReferenceElement(id) {
        return this.container.querySelector(`#${id}`);
    }
    clearFocus() {
        this.container.querySelectorAll(`.${this.options.className}.${FOCUS_CSS}`)
            .forEach((el) => el.classList.remove(FOCUS_CSS));
    }
    getHighlights() {
        const highlights = Object.values(this.highlights);
        highlights.sort((a, b) => {
            return a.range.compareBoundaryPoints(Range.START_TO_START, b.range);
        });
        return highlights;
    }
    get document() {
        if (!this.container.ownerDocument) {
            throw new Error('highlighter container is not mounted to a document!');
        }
        return this.container.ownerDocument;
    }
    onClick(selection) {
        const { onClick } = this.options;
        if (onClick) {
            const range = getRange(selection);
            const highlight = Object.values(this.highlights)
                .find((other) => other.intersects(range));
            onClick(highlight);
        }
    }
    onSelect(selection) {
        const { onSelect } = this.options;
        snapSelection(selection, this.options);
        if (onSelect) {
            const range = getRange(selection);
            const highlights = Object.values(this.highlights)
                .filter((other) => other.intersects(range));
            if (highlights.length === 0) {
                const highlight = new Highlight(range, rangeContentsString(range));
                onSelect(highlights, highlight);
            }
            else {
                onSelect(highlights);
            }
        }
    }
}
//# sourceMappingURL=Highlighter.js.map