import Highlighter from './Highlighter';
import * as xpath from './serializationStrategies/XpathRangeSelector/xpath';
import SerializedHighlight from './SerializedHighlight';
describe('isLoadable', () => {
    let highlighter = new Highlighter(document.createElement('div'));
    let serialized;
    let getReferenceElement;
    let getFirstByXPath;
    beforeEach(() => {
        highlighter = new Highlighter(document.createElement('div'));
        serialized = new SerializedHighlight({
            content: 'content',
            endContainer: 'end',
            endOffset: 0,
            id: 'id',
            referenceElementId: 'id',
            startContainer: 'start',
            startOffset: 0,
            type: 'XpathRangeSelector',
        });
        getReferenceElement = jest.spyOn(highlighter, 'getReferenceElement');
        getFirstByXPath = jest.spyOn(xpath, 'getFirstByXPath');
    });
    afterEach(() => {
        getReferenceElement.mockRestore();
        getFirstByXPath.mockRestore();
    });
    test(`returns false if reference element can't be found`, () => {
        getReferenceElement.mockImplementation(() => null);
        getFirstByXPath.mockImplementation(() => document.createElement('div'));
        expect(serialized.isLoadable(highlighter)).toEqual(false);
    });
    test(`returns false if start container can't be found`, () => {
        getReferenceElement.mockImplementation(() => document.createElement('div'));
        getFirstByXPath.mockImplementation((path) => path === 'end' ? document.createElement('div') : null);
        expect(serialized.isLoadable(highlighter)).toEqual(false);
    });
    test(`returns false if end container can't be found`, () => {
        getReferenceElement.mockImplementation(() => document.createElement('div'));
        getFirstByXPath.mockImplementation((path) => path === 'start' ? document.createElement('div') : null);
        expect(serialized.isLoadable(highlighter)).toEqual(false);
    });
    test(`returns true if all nodes are found`, () => {
        getReferenceElement.mockImplementation(() => document.createElement('div'));
        getFirstByXPath.mockImplementation(() => document.createElement('div'));
        expect(serialized.isLoadable(highlighter)).toEqual(true);
    });
});
//# sourceMappingURL=SerializedHighlight.test.js.map