import Highlighter from './Highlighter';
import * as xpath from './serializationStrategies/XpathRangeSelector/xpath';
import SerializedHighlight from './SerializedHighlight';

describe('isLoadable', () => {
  let highlighter: Highlighter = new Highlighter(document.createElement('div'));
  let serialized: SerializedHighlight;
  let getReferenceElement: jest.MockInstance<Highlighter['getReferenceElement']>;
  let getFirstByXPath: jest.MockInstance<() => void>;

  beforeEach(() => {
    highlighter = new Highlighter(document.createElement('div'));
    serialized = new SerializedHighlight({
      content: 'content',
      endContainer: 'end',
      endOffset: 0,
      id: 'id',
      referenceElementId: 'id',
      startContainer: 'start',
      startOffset: 0,
      type: 'XpathRangeSelector',
    });

    getReferenceElement = jest.spyOn(highlighter, 'getReferenceElement');
    getFirstByXPath = jest.spyOn(xpath, 'getFirstByXPath');
  });

  afterEach(() => {
    getReferenceElement.mockRestore();
    getFirstByXPath.mockRestore();
  });

  test(`returns false if reference element can't be found`, () => {
    getReferenceElement.mockImplementation(() => null);
    getFirstByXPath.mockImplementation(() => document.createElement('div'));

    expect(serialized.isLoadable(highlighter)).toEqual(false);
  });

  test(`returns false if start container can't be found`, () => {
    getReferenceElement.mockImplementation(() => document.createElement('div'));
    getFirstByXPath.mockImplementation((path: string) => path === 'end' ? document.createElement('div') : null);

    expect(serialized.isLoadable(highlighter)).toEqual(false);
  });

  test(`returns false if end container can't be found`, () => {
    getReferenceElement.mockImplementation(() => document.createElement('div'));
    getFirstByXPath.mockImplementation((path: string) => path === 'start' ? document.createElement('div') : null);

    expect(serialized.isLoadable(highlighter)).toEqual(false);
  });

  test(`returns true if all nodes are found`, () => {
    getReferenceElement.mockImplementation(() => document.createElement('div'));
    getFirstByXPath.mockImplementation(() => document.createElement('div'));

    expect(serialized.isLoadable(highlighter)).toEqual(true);
  });
});
